'use strict';

var language = require('@codemirror/language');
var lint = require('@codemirror/lint');
var common = require('@lezer/common');
var javascript = require('@lezer/javascript');
var lezer = require('@surrealdb/lezer');
var compareVersions = require('compare-versions');

const surrealqlLanguage = language.LRLanguage.define({
    name: "surrealql",
    parser: lezer.parser.configure({
        props: [
            language.indentNodeProp.add({
                Object: language.continuedIndent({ except: /^\s*}/ }),
                Array: language.continuedIndent({ except: /^\s*]/ }),
            }),
            language.foldNodeProp.add({
                "Object Array CombinedResult": language.foldInside,
            }),
        ],
        wrap: common.parseMixed((node) => {
            return node.name === "JavaScriptContent" ? { parser: javascript.parser } : null;
        }),
    }),
    languageData: {
        closeBrackets: { brackets: ["[", "{", '"', "'", "("] },
        indentOnInput: /^\s*[\]}]$/,
        commentTokens: { line: "--", block: { open: "/*", close: "*/" } },
    },
});
const scopeMap = new Map([
    ["permission", "PermissionInput"],
    ["index", "IndexInput"],
    ["combined-results", "CombinedResults"],
    ["syntax", "Syntax"],
]);
/**
 * A linter that checks if the syntax tree uses features that are not available in the given SurrealDB version
 *
 * @param version The SurrealDB version to check against (e.g. "2.0.0")
 */
function surrealqlVersionLinter(version) {
    if (!compareVersions.validate(version)) {
        throw new Error(`Invalid SurrealDB version: ${version}`);
    }
    return lint.linter((view) => {
        const diagnostics = [];
        language.syntaxTree(view.state)
            .cursor()
            .iterate((node) => {
            if (node.from === node.to) {
                return;
            }
            const sinceVersionProp = node.type.prop(lezer.sinceProp);
            const untilVersionProp = node.type.prop(lezer.untilProp);
            if (sinceVersionProp && compareVersions.compareVersions(version, sinceVersionProp) < 0) {
                diagnostics.push({
                    from: node.from,
                    to: node.to,
                    severity: "error",
                    message: `This syntax is only available on SurrealDB ${sinceVersionProp} and up`,
                });
            }
            if (untilVersionProp && compareVersions.compareVersions(version, untilVersionProp) >= 0) {
                diagnostics.push({
                    from: node.from,
                    to: node.to,
                    severity: "error",
                    message: `This syntax is only available until SurrealDB ${untilVersionProp}`,
                });
            }
        });
        return diagnostics;
    });
}
/**
 * The CodeMirror extension used to add support for the SurrealQL language
 *
 * @param scope Limit the scope of the highlighting
 */
function surrealql(scope) {
    if (!scope) {
        return new language.LanguageSupport(surrealqlLanguage);
    }
    const scopeId = scopeMap.get(scope);
    if (!scopeId) {
        throw new Error(`Unknown language scope: ${scope}`);
    }
    return new language.LanguageSupport(surrealqlLanguage.configure({ top: scopeId }));
}

exports.surrealql = surrealql;
exports.surrealqlLanguage = surrealqlLanguage;
exports.surrealqlVersionLinter = surrealqlVersionLinter;
